/*****************************************************************************
 *
 * PPS (Peripheral Pin Select) Lab - Echoes typed keys back to HyperTerminal
 *
 *****************************************************************************
 * FileName:        pps.c
 * Dependencies:    p24FJ64GA004.h
 * Processor:		PIC24FJ64GA004
 * IDE :       	    MPLAB v8.10 or higher
 * Compiler :       MPLAB C30 v3.10 or higher
 * HW platform :    EXPLORER16 (DM240001)
 * Plug-In Module : PIC24FJ64GA004 PIM Plug-In Module (MA240013)
 * Company:         Microchip Technology Incorporated
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the "Company") is intended and supplied to you, the Company's
 * customer, for use solely and exclusively with products manufactured
 * by the Company. 
 *
 * The software is owned by the Company and/or its supplier, and is 
 * protected under applicable copyright laws. All rights are reserved. 
 * Any use in violation of the foregoing restrictions may subject the 
 * user to criminal sanctions under applicable laws, as well as to 
 * civil liability for the breach of the terms and conditions of this 
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES, 
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT, 
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Cecic, D          18/Jan/07		MPLAB7.51, C30v2.05 + updated .h and .gld file
 * Brant Ivey			7/3/07		Updated for v2.0 of class
 * Alain Sorin        2/Nov/07      Updated for v3.10 of 204_ADV class
 *                                  (removed auto_psv warning, UART comments corrected)
 * Alain Sorin       28/feb/09      updated to v4.00 of MCU_3122 (RTC class renamed)
 *                                  added macro to automatically compute BRGVAL from Fosc
 **************************************************************************************/

#include <p24FJ64GA004.h>

/*** Macro definition *****************************************************************/
// This macro computes the closest BRG value (rounding) based on  BAUDRATE and FCY

#define round(x) 		(int)(((float)(x))>0?(float)(x)+0.5:(float)(x)-0.5)
#define Fosc   			8000000
#define FCY   			Fosc/2
#define BAUDRATE        9600                   
#define BRGVAL          round((FCY/BAUDRATE/(float)16)-1)

//****************************************************************************************
//	STEP 1:
//	Set up the Configuration Bits in the file using the _CONFIGx macros
//  Macros and bit configuration labels are defined in the p24FJ64GA004.h file
//  Details about bit configuration registers in PIC24FJ64GA004 datasheet, section 23
//	(JTAG=Disabled, WATCHDOG=Disabled, EMUC/EMUD share PGC1/PGD1, PriOSC=HS, IOLOCK=1 Way)
//***************************************************************************************/

// ### your code here ###	// ###  configure _CONFIG1
// ### your code here ###	// ###  configure _CONFIG2


/*** Function Prototypes **************************************************************/
void ioMap(void);			// Assign RPn pin(s) to input/output functions for this lab
void unlockIO(void);		// Unlock sequence for RPn configuration
void lockIO(void);			// Lock sequence for RPn configuration
void UARTInit(void);		// Initiallize UART for 9600/N/8/1/HW Flow Control
void IOInit(void);			// Initialize switch and LED I/O


/*** main() Function********************************************************************/
int main(void)
{
	ioMap();				// Configure the remappable I/O pins
	UARTInit();      		// Initialize UART2
	IOInit();				// Initialize Switch and LED I/O
	
	IEC1bits.U2RXIE = 1;	// Enable Uart2 RX Interrupts
	
	while(1);
}

/*********************************************************************************** 
 * Function: ioMap()
 *
 * Preconditions: None.
 *
 * Overview: This executes the necessary process to map the RPn pins to UART 2 for the
 *				PIC24FJ64GA004-PIM installed on Explorer16
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/
void ioMap()
{
	//*************************************************
	//	STEP 2:     Unlock the Reprogrammable Pin Mechanism
	//*************************************************/
	// ### your code here ###	               	// ###

	//*************************************************
	//	STEP 3:     Map RP19 pin to input function U2RX
	//*************************************************/
	// ### your code here ###	               	// ###
	
	//*************************************************
	//	STEP 4:     Map RP20 pin to input function U2CTS
	//*************************************************/
	// ### your code here ###	               	// ###

	//*************************************************
	//	STEP 5:     Map Output Function U2TX to RP25
	//*************************************************/
	// ### your code here ###	               	// ###

	//*************************************************
	//	STEP 6:     Map Output Function U2RTS to RP21
	//*************************************************/
	// ### your code here ###	               	// ###
	
	//*************************************************
	//	STEP 7:     Lock the Reprogrammable Pin Mechanism
	//*************************************************/
	// ### your code here ###	               	// ###
}
 
/*********************************************************************************** 
 * Function: lockIO
 *
 * Preconditions: None.
 *
 * Overview: This executes the necessary process to set the IOLOCK bit to lock
 * I/O mapping from being modified.
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/
void lockIO(){

asm volatile ("mov #OSCCON,w1 \n"
				"mov #0x46, w2 \n"
				"mov #0x57, w3 \n"
				"mov.b w2,[w1] \n"
				"mov.b w3,[w1] \n"
				"bset OSCCON, #6":::"w1","w2","w3");
}

/*****************************************************************************
 * Function: unlockIO
 *
 * Preconditions: None.
 *
 * Overview: This executes the necessary process to clear the IOLOCK bit to 
 * allow I/O mapping to be modified.
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/
void unlockIO()
{
asm volatile ("mov #OSCCON,w1 \n"
				"mov #0x46, w2 \n"
				"mov #0x57, w3 \n"
				"mov.b w2,[w1] \n"
				"mov.b w3,[w1] \n"
				"bclr OSCCON, #6":::"w1","w2","w3");
}

/*********************************************************************************
*
*	UARTinit() - Initialize UART2 for 9600/8/N/1/HW Flow control
*
**********************************************************************************/
void UARTInit()
{
	TRISCbits.TRISC9 = 0;	// RC9 on 24FJ64GA004 connected to UART TX = output function
	TRISCbits.TRISC5 = 0;	// RC5 on 24FJ64GA004 connected to UART RTS = output function

	//********************************************************************************
	//  Load the count to get 9600 baudrate
	//	BRG = Fcy/(16*BaudRate)-1  where Fcy = Fosc/2 = 4MHz
	//********************************************************************************/
	U2BRG = BRGVAL; 		// automatic rounding macro computation = 0x19; 

	//********************************************************************************
	//  Hardware Flow Control Mode, No Loop-Back
	// 	No Auto-Baud, No Rx-Inversion
	//	Low BaudRate, 1 Stop-bit
	//	8-bit, no parity 
	//********************************************************************************/
	U2MODE = 0x8200; 		// 0b1000 0010 0000 0000

	//********************************************************************************
	//  Tx Interrupt to interrupt when at least one location is free in Tx buffer
	//	No Tx Inversion, Disable Sync Break
	//	Enable Transmit, Disable Address Detect
	//	Interrupt on every receive. 
	//********************************************************************************/
	U2STA = 0x0400; 		// 0b0000 0100 0000 0000

	IFS1bits.U2RXIF = 0;		// Clear U2RX_Int flag
}

/*********************************************************************************
*
*	IOInit() - Initialize Switch and LED I/O for the project
*
**********************************************************************************/
void IOInit()
{
	LATAbits.LATA10 = 0;	
	TRISAbits.TRISA10 = 0;	// RA10 as output for LED D3 on EXPLORER16
}

/*********************************************************************************
*
*	_U2RXInterrupt - Echo character back to Hyperterminal or another terminal SW
*
**********************************************************************************/
// shadow attribute is used here to reduce interrupt save and restore overhead

void __attribute__((interrupt, shadow, no_auto_psv)) _U2RXInterrupt()
{
	IFS1bits.U2RXIF = 0;

	while(!U2STAbits.TRMT);

    if( U2STAbits.OERR )                 // if overrun, clear OERR flag
        U2STAbits.OERR = 0;				 // according to datasheet paragraph 16.5

	U2TXREG = U2RXREG;					// echo received character

	__builtin_btg((unsigned int*)&LATA,10); // switch ON and OFF LED D3
}
