/*****************************************************************************
 *
 * Real Time Clock Calender
 *
 *****************************************************************************
 * FileName:        rtcc.h
 * Dependencies:    
 * Processor:       PIC24FJ128GA010
 * MPLAB:           MPLAB v8.10 or higher
 * Compiler:       	C30 V3.10 or higher 
 * Company:         Microchip Technology Incorporated
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the "Company") is intended and supplied to you, the Company's
 * customer, for use solely and exclusively with products manufactured
 * by the Company. 
 *
 * The software is owned by the Company and/or its supplier, and is 
 * protected under applicable copyright laws. All rights are reserved. 
 * Any use in violation of the foregoing restrictions may subject the 
 * user to criminal sanctions under applicable laws, as well as to 
 * civil liability for the breach of the terms and conditions of this 
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES, 
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT, 
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Chris Valenti		05-26-05	...	
 * Ross Fosler			06-06-2005	Several changes
 * Anton Alkhimenok     10-21-2005  Get/Set functions
 * Alain Sorin          28/Feb/09   updated to v4.00 of MCU_3122 RTC class
 *****************************************************************************/

#define	EECON2	NVMKEY 

// Setup clock
extern void RTCCInit(void);
// Must be called periodically to refresh time and date strings
extern void RTCCProcessEvents(void);
// Write changed time and date to clock registers 
extern void RTCCSet(void);
// Unlock access to clock and read time and date 
extern void RTCCUnlock(void);

void AlarmSet(void);

// Union to access rtcc registers
typedef union tagRTCC {
	struct {
		unsigned char sec;
		unsigned char min;
		unsigned char hr;
		unsigned char wkd;
		unsigned char day;
		unsigned char mth;
		unsigned char yr;
	};
	struct {
		unsigned int prt00;
		unsigned int prt01;
		unsigned int prt10;
		unsigned int prt11;
	};
} RTCC;

// Gets time and date 
extern RTCC _time;

// Gets time and date second time to detect overflow
extern RTCC _time_chk;

// _time_str and _date_str contain current time and date strings
extern char _time_str[16]; // Tue 07:12:34
extern char _date_str[16]; // Feb 17, 2009

// Macros to initiate and set RTCC
#define mRTCCInit()		RTCCInit()
#define	mRTCCSet()		RTCCSet()
#define mRTCCOn()		RCFGCALbits.RTCEN = 1;
#define	mRTCCOff()		RCFGCALbits.RTCEN = 0;
#define mRTCCUnlock()	RTCCUnlock();
#define mRTCCLock()		RCFGCALbits.RTCWREN = 0;

#define mRTCCGetSec()	_time_chk.sec
#define mRTCCGetMin()	_time_chk.min
#define mRTCCGetHour()	_time_chk.hr
#define mRTCCGetWkDay()	_time_chk.wkd
#define mRTCCGetDay()	_time_chk.day
#define mRTCCGetMonth()	_time_chk.mth
#define	mRTCCGetYear()	_time_chk.yr

#define	mALARMSet()		AlarmSet()
/*****************************************************************************
 * EOF
 *****************************************************************************/
