/*****************************************************************************
 *
 * State mashine to display/setup Real Time Clock Calender
 *
 *****************************************************************************
 * FileName:        tbanner.c
 * Dependencies:    system.h    
 * Processor:		PIC24FJ128GA010
 * IDE :       	    MPLAB v8.10 or higher
 * Compiler :       MPLAB C30 v3.10 or higher
 * HW platform :    EXPLORER16 (DM240001)
 * Plug-In Module : PIC24FJ128GA010 PIM Plug-In Module (MA240011)
 * Company:         Microchip Technology Incorporated
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the "Company") is intended and supplied to you, the Company's
 * customer, for use solely and exclusively with products manufactured
 * by the Company. 
 *
 * The software is owned by the Company and/or its supplier, and is 
 * protected under applicable copyright laws. All rights are reserved. 
 * Any use in violation of the foregoing restrictions may subject the 
 * user to criminal sanctions under applicable laws, as well as to 
 * civil liability for the breach of the terms and conditions of this 
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES, 
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT, 
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * XXX                  XXX         ...	
 * Anton Alkhimenok     10-21-2005  Get/Set related changings
 * Alain Sorin          7/Dec/07    Updated for v3.10 of 204_ADV class 
 * Alain sorin          28/Feb/09   Updated to v4.00 of MCU_3122 class
 *****************************************************************************/

#include "system.h"

unsigned char 	_uTBannerState;
char * 	        _pTBanner;          // Pointer to character dislayed.
unsigned char 	_uTBannerLen;       // Position displayed.
unsigned int 	_uTBannerCharWait;
unsigned char 	_uTBannerCurPos;    // Current field selected.


/*****************************************************************************
 * Clock setup related definitions.
 *****************************************************************************/
#define TBNR_BLINK_PERIOD   20000
#define TBNR_POS_MAX        6
#define TBNR_POS_WKDAY      0
#define TBNR_POS_HOUR       1
#define TBNR_POS_MIN        2
#define TBNR_POS_SEC        3
#define TBNR_POS_MONTH      4
#define TBNR_POS_DAY        5
#define TBNR_POS_YEAR       6

/*****************************************************************************
 * Function: TBannerInit
 *
 * Precondition: None.
 *
 * Overview: The function setup state mashine to display clock.
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/
void TBannerInit(void)
{
	_uTBannerLen = LCD_DISPLAY_LEN;
	_uTBannerState = 1;
	_uTBannerCharWait = 2;
	_pTBanner = _time_str;
    _uTBannerCurPos = TBNR_POS_HOUR;
}


/*****************************************************************************
 * Function: TBannerProcessEvents
 *
 * Precondition: TBannerInit must be called before.
 *
 * Overview: This is a state mashine to display time and date strings and
 * show the blinking cursor in SETUP mode.
 *
 * Input: None.
 *
 * Output: None.
 *
 *****************************************************************************/
void TBannerProcessEvents(void)
{
   
	switch(_uTBannerState){
		case 1: 			// Wait to put a char
		case 4:
			if (_uTBannerCharWait) _uTBannerCharWait--;
			else {
				_uTBannerState++;
				_uTBannerCharWait = 2;
			}
			break;
		case 5:
		case 2:				// Put a char on the LCD
			if (!mLCDIsBusy()) {
				mLCDPutChar(*_pTBanner);
				_pTBanner++;
				_uTBannerLen--;
				if (!_uTBannerLen) _uTBannerState++;
				else _uTBannerState--;
			}
			break;
		case 3:
			if (!mLCDIsBusy()) {
				mLCDPutCmd(0xC0);
				_uTBannerLen = LCD_DISPLAY_LEN;

                 RTCCProcessEvents();

   				_pTBanner = _date_str;
				_uTBannerState++;
			}
			break;
		case 6:
			if (!mLCDIsBusy()) {
				mLCDHome();
				_uTBannerState = 1;
				_uTBannerLen = LCD_DISPLAY_LEN;
   				_pTBanner = _time_str;
			}
			break;
		default:
			_uTBannerState = 0;
			break;
	}	
}
/*****************************************************************************
 * EOF
 *****************************************************************************/
