/*****************************************************************************
 *
 * LCD Driver for PIC24.
 *
 *****************************************************************************
 * FileName:        lcd.c
 * Dependencies:    p24FJ128GA010.h
 * Processor:		PIC24FJ128GA010
 * IDE :       	    MPLAB v8.10 or higher
 * Compiler :       MPLAB C30 v3.10 or higher
 * HW platform :    EXPLORER16 (DM240001)
 * Plug-In Module : PIC24FJ128GA010 PIM Plug-In Module (MA240011)         
 * Company:         Microchip Technology Incorporated
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the "Company") is intended and supplied to you, the Company's
 * customer, for use solely and exclusively with products manufactured
 * by the Company. 
 *
 * The software is owned by the Company and/or its supplier, and is 
 * protected under applicable copyright laws. All rights are reserved. 
 * Any use in violation of the foregoing restrictions may subject the 
 * user to criminal sanctions under applicable laws, as well as to 
 * civil liability for the breach of the terms and conditions of this 
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES, 
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT, 
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *
 * A simple LCD driver for LCDs interface through the PMP
 * 
 *
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Vidyadhar            30/June/06  ...	
 * Brant Ivey           7/3/07      Updated for v2.0 of class
 * Alain Sorin          2/Nov/07    Updated for v3.10 of 204_ADV class 
 * Stu Chandler         12/20/08    Updated for both 3V & 5V LCds
 * Alain Sorin          28/Feb/09   Updated for v4.00 of MCU_3122 class (renamed)
 *****************************************************************************/

#include	<p24FJ128GA010.h>

// Constants for LCD Delay routines.  

// Define Fosc for HS Oscillator (HS)
// Delay loop is 1uS = 4 Tcy = 8 Tosc
#define Fosc 8000000                    // HS Fosc = 8MHz

// Define a fast instruction execution time in terms of loop time
#define LCD_F_INSTR     (unsigned int)(Fosc/160000)     // 50uS

// Define a slow instruction execution time in terms of loop time
#define LCD_S_INSTR     (unsigned int)(Fosc/4000)       //  2mS

// Define the startup time for the LCD in terms of loop time 
#define LCD_STARTUP     (unsigned int)(Fosc/160)        // 50mS

unsigned int	_uLCDloops;


LCDInit()
{
	PMMODE = 0x3FF;
	PMCON = 0x8303;
	PMAEN = 0x0001;

	_uLCDloops = LCD_STARTUP;
	Wait(_uLCDloops);

	_uLCDloops = LCD_F_INSTR;
	PMDIN1 = 0b00111000;			// Set the default function
	Wait(_uLCDloops);

	_uLCDloops = LCD_STARTUP;
	Wait(_uLCDloops);

	_uLCDloops = LCD_F_INSTR;
	PMDIN1 = 0b00001100;
	Wait(_uLCDloops);

	_uLCDloops = LCD_STARTUP;
	Wait(_uLCDloops);

	_uLCDloops = LCD_S_INSTR;
	PMDIN1 = 0b00000001;			// Clear the display
	Wait(_uLCDloops);

	_uLCDloops = LCD_STARTUP;
	Wait(_uLCDloops);

	_uLCDloops = LCD_S_INSTR;
	PMDIN1 = 0b00000110;			// Set the entry mode
	Wait(_uLCDloops);

	LCDClear();
	LCDHome();
}

LCDHome()
{
	_uLCDloops = LCD_S_INSTR;
	PMADDR = 0x0000;
	PMDIN1 = 0b00000010;
	while(_uLCDloops)
	_uLCDloops--;
}

LCDL1Home()
{
	_uLCDloops = LCD_S_INSTR;
	PMADDR = 0x0000;
	PMDIN1 = 0b10000000;
	while(_uLCDloops)
	_uLCDloops--;
}

LCDL2Home()
{
	_uLCDloops = LCD_S_INSTR;
	PMADDR = 0x0000;
	PMDIN1 = 0b11000000;
	while(_uLCDloops)
	_uLCDloops--;
}

LCDClear()
{
	_uLCDloops = LCD_S_INSTR;
	PMADDR = 0x0000;
	PMDIN1 = 0b00000001;
	while(_uLCDloops)
	_uLCDloops--;
}

LCDPut(char A)
{
	_uLCDloops = LCD_F_INSTR;
	PMADDR = 0x0001;
	PMDIN1 = A;
	while(_uLCDloops)
	_uLCDloops--;
	Nop();
	Nop();
	Nop();
	Nop();
}

Wait(unsigned int B)
{
	while(B)
	B--;
}

