/*****************************************************************************
 *
 * CRC Demo for PIC24.
 *
 *****************************************************************************
 * FileName:        main.c
 * Dependencies:    p24fJ128GA010.h
 * Processor:		PIC24FJ128GA010
 * IDE :       	    MPLAB v8.10 or higher
 * Compiler :       MPLAB C30 v3.10 or higher
 * HW platform :    EXPLORER16 (DM240001)
 * Plug-In Module : PIC24FJ128GA010 PIM Plug-In Module (MA240011)        
 * Company:         Microchip Technology Incorporated
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the "Company") is intended and supplied to you, the Company's
 * customer, for use solely and exclusively with products manufactured
 * by the Company. 
 *
 * The software is owned by the Company and/or its supplier, and is 
 * protected under applicable copyright laws. All rights are reserved. 
 * Any use in violation of the foregoing restrictions may subject the 
 * user to criminal sanctions under applicable laws, as well as to 
 * civil liability for the breach of the terms and conditions of this 
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES, 
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT, 
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Vidyadhar           30/June/06	...	
 * Brant Ivey			7/3/07		Updated for v2.0 of class
 * Alain Sorin          2/Nov/07    Updated for v3.10 of 204_ADV class
 * Alain Sorin         28/Feb/09    Updated for v4.00 of MCU_3122 class (renamed)
 *                                  main() function modified to operate in loop
 *                                  CRC & UART are RESET to test OK & BAD conditions
 ********************************************************************************/

#include <p24fJ128GA010.h>    

// Setup configuration bits
_CONFIG1(JTAGEN_OFF & GCP_OFF & GWRP_OFF & BKBUG_OFF & ICS_PGx2 & FWDTEN_OFF)
_CONFIG2(IESO_OFF & FCKSM_CSDCMD & OSCIOFNC_ON &  FNOSC_PRI & POSCMOD_HS)


unsigned int CRCResult;
unsigned int DataCRC[12];

CRCInit()
{
	//********************************************************************
	//	STEP 1:
	//	In CRCCON, Configure The Polynomial Length (PLEN)
	//  for the polynomial x^16 + x^15 + x^2 + 1 
	//*******************************************************************/
	// ### your code here ###	// ### 

	//********************************************************************
	//	STEP 2:
	//	In CRCXOR, Configure For The Polynomial x^16 + x^15 + x^2 + 1
	//********************************************************************/
	// ### your code here ###	// ### 
	
	//********************************************************************
	//	STEP 3:
	//	Clear CRCWDAT
	//********************************************************************/
	// ### your code here ###	// ### 	

} // End of init_CRC()


CalculateCRC(unsigned int *Src, char Len)
{
	int i, CRCTemp;
	char Temp=0;

	CRCWDAT=0x0000;

	for(i=0;i<Len;i++)
	{	
		CRCDAT=Src[i];				// Input received data

		if(CRCCONbits.CRCFUL)
		{

			//********************************************************************
			//	STEP 4:
			//	In CRCCON, Enable the CRC Generator 
			//********************************************************************/
			// ### your code here ###	// ### 

		}
		while(CRCCONbits.CRCFUL);	// Wait for available FIFO register
	}
	CRCDAT = 0x0000;  				// Pad last word with 0's

	while (!IFS4bits.CRCIF);		// check for interrupt flag bit
	IFS4bits.CRCIF = 0;             // clear CRC interrupt flag

	while(!CRCCONbits.CRCMPT);		// wait for CRC shifter to clear FIFO
	CRCResult = CRCWDAT;			// Ouptut result
	CRCWDAT=0;
}


DisplayWaitCRC()
{
	LCDClear();
	LCDHome();       // Set cursor to Line 1 of LCD
	LCDPut('W');
	LCDPut('a');
	LCDPut('i');
	LCDPut('t');
	LCDPut('i');
	LCDPut('n');
	LCDPut('g');
	LCDPut(' ');
	LCDPut('f');
	LCDPut('o');
	LCDPut('r');
	LCDPut(' ');
	LCDPut('D');
	LCDPut('a');
	LCDPut('t');
	LCDPut('a');
}

DisplayCRCMatch()
{
	LCDClear();
	LCDHome();       // Set cursor to Line 1 of LCD
	LCDPut('C');
	LCDPut('R');
	LCDPut('C');
	LCDPut(' ');
	LCDPut('v');
	LCDPut('e');
	LCDPut('r');
	LCDPut('i');
	LCDPut('f');
	LCDPut('i');
	LCDPut('e');
	LCDPut('d');
	LCDPut(':');
	LCDPut('O');
	LCDPut('K');
}

DisplayCRCNoMatch()
{
	LCDClear();
	LCDHome();       // Set cursor to Line 1 of LCD
	LCDPut('C');
	LCDPut('R');
	LCDPut('C');
	LCDPut(' ');
	LCDPut('v');
	LCDPut('e');
	LCDPut('r');
	LCDPut('i');
	LCDPut('f');
	LCDPut('i');
	LCDPut('e');
	LCDPut('d');
	LCDPut(':');
	LCDPut('B');
	LCDPut('A');
	LCDPut('D');
}

main()
{
	LCDInit();           // Initialize LCD module.

	CRCInit();			 // Initialize CRC module. You write this function
	
    RS232Init();		 // Initialize UART Communications

    
	while(1)
	{
	    
		DisplayWaitCRC();	 // Display waiting message to LCD.    
	    
	    GetRS232(DataCRC, 11);  // Get data message from UART
	
	    CalculateCRC(DataCRC, 11); // Calculate CRC with received data
	
		// Display result to LCD.
	    if (CRCResult == 0)
		  DisplayCRCMatch();
		else
		  DisplayCRCNoMatch();
		
		unsigned long int i;
		for (i=0;i<1000000;i++) Nop() ;

		CRCInit();			            // Re-Initialize CRC module
		
		U2MODEbits.UARTEN = 0;			// Disable UART (clean up remaining FIFO bytes if any)
		U2STAbits.UTXEN = 0;			// Disable Transmit
		U2MODEbits.UARTEN = 1;			// Re-enable UART
		U2STAbits.UTXEN = 1;			// Re-enable Transmit
	}	
}
/********************************************************************************************/
