/*****************************************************************************
 *
 * UART Communications for PIC24.
 *
 *****************************************************************************
 * FileName:        rs232.c
 * Dependencies:    p24FJ128GA010.h  
 * Processor:		PIC24FJ128GA010
 * IDE :       	    MPLAB v8.10 or higher
 * Compiler :       MPLAB C30 v3.10 or higher
 * HW platform :    EXPLORER16 (DM240001)
 * Plug-In Module : PIC24FJ128GA010 PIM Plug-In Module (MA240011)       
 * Company:         Microchip Technology Incorporated
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the "Company") is intended and supplied to you, the Company's
 * customer, for use solely and exclusively with products manufactured
 * by the Company. 
 *
 * The software is owned by the Company and/or its supplier, and is 
 * protected under applicable copyright laws. All rights are reserved. 
 * Any use in violation of the foregoing restrictions may subject the 
 * user to criminal sanctions under applicable laws, as well as to 
 * civil liability for the breach of the terms and conditions of this 
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES, 
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED 
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT, 
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR 
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 *
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Vidyadhar		   30/June/06	     ...	
 * Alain Sorin          2/Nov/07    Updated for v3.10 of 204_ADV class
 * Alain Sorin         28/Feb/09    Updated for v4.00 of MCU_3122 class (renamed)
 *                                  added macro to automatically compute BRGVAL from Fosc     
 **************************************************************************************/

#include <p24FJ128GA010.h>

/*** Macro definition *****************************************************************/
// This macro computes the closest BRG value (rounding) based on  BAUDRATE and FCY

#define round(x) 		(int)(((float)(x))>0?(float)(x)+0.5:(float)(x)-0.5)
#define Fosc   			8000000
#define FCY   			Fosc/2
#define BAUDRATE        9600                   
#define BRGVAL          round((FCY/BAUDRATE/(float)16)-1)

/*************************************************************************************/

RS232Init()
{
// 9600, N , 8, 1, no flow control with Fosc = 8 MHz

	TRISFbits.TRISF5 = 0;

	U2MODE = 0x0800;
	U2STA = 0x0000;
	U2BRG = BRGVAL ; 				// 0x19 for 9600bauds @ Fosc = 8MHz					
	U2MODEbits.UARTEN = 1;			// Enable UART
	U2STAbits.UTXEN = 1;			// Enable Transmit
}


GetRS232(int *a,char Len)
{
char i;
char p, q, r, s;

	for(i=0;i<Len;i++)
	{
	  while(!U2STAbits.URXDA);
	  p = U2RXREG;
	  U2TXREG = p;
	  while(!U2STAbits.TRMT);

	  while(!U2STAbits.URXDA);
	  q = U2RXREG;
	  U2TXREG = q;
	  while(!U2STAbits.TRMT);

	  while(!U2STAbits.URXDA);
	  r = U2RXREG;
	  U2TXREG = r;
	  while(!U2STAbits.TRMT);

	  while(!U2STAbits.URXDA);
	  s = U2RXREG;
	  U2TXREG = s;
	  while(!U2STAbits.TRMT);

      if(p>0x40)
		p = p + 0x09;
	    p = p & 0x0f;

      if(q>0x40)
		q = q + 0x09;
	    q = q & 0x0f;

      if(r>0x40)
		r = r + 0x09;
	    r = r & 0x0f;

      if(s>0x40)
		s = s + 0x09;
	    s = s & 0x0f;

 	    a[i] = (int) p;
	    a[i] = a[i] << 4;
	    a[i] = a[i] | (int) q;	
	    a[i] = a[i] << 4;
	    a[i] = a[i] | (int) r;	
	    a[i] = a[i] << 4;
	    a[i] = a[i] | (int) s;	
	}
}

PutRS232(char P)
{
    while(!U2STAbits.TRMT);
 	U2TXREG = P;
}
